@@ -1,6 +1,5 @@
-/* Time-stamp: <2005-12-11 17:54:30 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_itdb.c,v 1.40 2005/12/11 09:44:35 jcsjcs Exp $
+|  $Id: display_itdb.c 1596 2007-06-26 14:33:01Z dforsi $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -38,23 +37,34 @@
 #include "details.h"
 #include "display_itdb.h"
 #include "display.h"
-#include "md5.h"
+#include "sha1.h"
 #include "file.h"
+#include "file_convert.h"
 #include "misc.h"
 #include "misc_track.h"
 #include "info.h"
 #include "prefs.h"
+#include "syncdir.h"
 
 
 /* A struct containing a list with available iTunesDBs. A pointer to
    this struct is stored in gtkpod_window as itdbs_head */
 static struct itdbs_head *itdbs_head = NULL;
 
+/* for convenience */
+struct itdbs_head *gp_get_itdbs_head (GtkWidget *gtkpod_win)
+{
+    g_return_val_if_fail (gtkpod_win, NULL);
+    return g_object_get_data (G_OBJECT (gtkpod_win),
+			      "itdbs_head");
+}
+
 void gp_itdb_extra_destroy (ExtraiTunesDBData *eitdb)
 {
     if (eitdb)
     {
-	md5_free_eitdb (eitdb);
+	sha1_free_eitdb (eitdb);
+	gp_itdb_pc_path_hash_destroy (eitdb);
 	g_free (eitdb);
     }
 }
@@ -97,7 +107,7 @@ void gp_track_extra_destroy (ExtraTrackData *etrack)
 	g_free (etrack->year_str);
 	g_free (etrack->pc_path_utf8);
 	g_free (etrack->hostname);
-	g_free (etrack->md5_hash);
+	g_free (etrack->sha1_hash);
 	g_free (etrack->charset);
 	g_free (etrack);
     }
@@ -115,11 +125,14 @@ ExtraTrackData *gp_track_extra_duplicate (ExtraTrackData *etr)
 	etr_dup->year_str = g_strdup (etr->year_str);
 	etr_dup->pc_path_locale = g_strdup (etr->pc_path_locale);
 	etr_dup->pc_path_utf8 = g_strdup (etr->pc_path_utf8);
+	etr_dup->converted_file = g_strdup (etr->converted_file);
 	etr_dup->thumb_path_locale = g_strdup (etr->thumb_path_locale);
 	etr_dup->thumb_path_utf8 = g_strdup (etr->thumb_path_utf8);
 	etr_dup->hostname = g_strdup (etr->hostname);
-	etr_dup->md5_hash = g_strdup (etr->md5_hash);
+	etr_dup->sha1_hash = g_strdup (etr->sha1_hash);
 	etr_dup->charset = g_strdup (etr->charset);
+	/* clear the pc_path_hashed flag */
+	etr_dup->pc_path_hashed = FALSE;
     }
     return etr_dup;
 }
@@ -134,6 +147,21 @@ iTunesDB *gp_itdb_new (void)
 }
 
 
+/* Free itdb and take care of dependencies */
+void gp_itdb_free (iTunesDB *itdb)
+{
+    if (space_get_ipod_itdb () == itdb)
+    {
+	space_set_ipod_itdb (NULL);
+    }
+
+    /* cancel all pending conversions */
+    file_convert_cancel_itdb (itdb);
+
+    itdb_free (itdb);
+}
+
+
     /* Add and initialize ExtraiTunesDBData if missing */
 void gp_itdb_add_extra (iTunesDB *itdb)
 {
@@ -149,12 +177,13 @@ void gp_itdb_add_extra (iTunesDB *itdb)
 	    (ItdbUserDataDuplicateFunc)gp_itdb_extra_duplicate;
 	eitdb->data_changed = FALSE;
 	eitdb->itdb_imported = FALSE;
+	gp_itdb_pc_path_hash_init (eitdb);
     }
 }
 
 
 /* Validate a complete @itdb (including tracks and playlists),
- * i.e. add the Extra*Data and validate the track entries */
+ * i.e. add the Extra*Data */
 void gp_itdb_add_extra_full (iTunesDB *itdb)
 {
     GList *gl;
@@ -242,7 +271,7 @@ void gp_track_add_extra (Track *track)
 Track *gp_track_add (iTunesDB *itdb, Track *track)
 {
     Track *result=NULL;
-    Track *oldtrack = md5_track_exists_insert (itdb, track);
+    Track *oldtrack = sha1_track_exists_insert (itdb, track);
 
     if(oldtrack)
     {
@@ -254,12 +283,16 @@ Track *gp_track_add (iTunesDB *itdb, Track *track)
     {
 	/* Make sure all strings are initialised -- that way we don't
 	   have to worry about it when we are handling the strings */
-	/* exception: md5_hash, hostname, charset: these may be NULL. */
+	/* exception: sha1_hash, hostname, charset: these may be NULL. */
 	gp_track_validate_entries (track);
 	itdb_track_add (itdb, track, -1);
+	/* add to filename hash */
+	gp_itdb_pc_path_hash_add_track (track);
+	/* add to background conversion if necessary */
+	file_convert_add_track (track);
 	result = track;
+	data_changed (itdb);
     }
-    data_changed (itdb);
     return result;
 }
 
@@ -270,10 +303,12 @@ Track *gp_track_add (iTunesDB *itdb, Track *track)
    playlist -- see gp_playlist_remove_track for details */
 void gp_track_remove (Track *track)
 {
-    /* currently only the details window may be accessing the tracks */
-    details_remove_track (track);
-    /* remove from database */
-    itdb_track_remove (track);
+    /* call gp_track_unlink() and itdb_track_free() instead of
+       itdb_track_remove() so we don't have to maintain two remove
+       functions separately. If something needs to be done before
+       removing the track do it in gp_track_unlink */
+    gp_track_unlink (track);
+    itdb_track_free (track);
 }
 
 
@@ -283,44 +318,91 @@ void gp_track_remove (Track *track)
    playlist -- see gp_playlist_remove_track for details */
 void gp_track_unlink (Track *track)
 {
-    /* currently only the details window may be accessing the tracks */
+    /* the details window may be accessing the tracks */
     details_remove_track (track);
+    /* cancel pending conversions */
+    file_convert_cancel_track (track);
+    /* remove from SHA1 hash */
+    sha1_track_remove (track);
+    /* remove from pc_path_hash */
+    gp_itdb_pc_path_hash_remove_track (track);
     /* remove from database */
     itdb_track_unlink (track);
 }
 
 
+/* Set a thumbnail and update ExtraTrackData (e.g. filename) */
+static gboolean gp_track_set_thumbnails_internal (Track *track,
+						  const gchar *filename,
+						  const guchar *image_data,
+						  gsize image_data_len)
+{
+    gboolean result = FALSE;
+    ExtraTrackData *etr;
+
+    g_return_val_if_fail (track, FALSE);
+
+    etr = track->userdata;
+    g_return_val_if_fail (etr, FALSE);
+
+    if (filename)
+    {
+	result = itdb_track_set_thumbnails (track, filename);
+    }
+    else if (image_data)
+    {
+	result = itdb_track_set_thumbnails_from_data (track,
+						      image_data,
+						      image_data_len);
+    }
+
+    g_free (etr->thumb_path_locale);
+    g_free (etr->thumb_path_utf8);
+
+    if (filename && (result == TRUE))
+    {
+	etr->thumb_path_locale = g_strdup (filename);
+	etr->thumb_path_utf8 = charset_to_utf8 (filename);
+    }
+    else
+    {
+	etr->thumb_path_locale = g_strdup ("");
+	etr->thumb_path_utf8 = g_strdup ("");
+    }
+
+    if (result == FALSE)
+    {
+	if (filename)
+	    gtkpod_warning (_("Failed to set cover art: '%s'\n"), filename);
+    }
+
+    return result;
+}
+
+/* Set a thumbnail and update data in ExtraTrackData */
+gboolean gp_track_set_thumbnails_from_data (Track *track,
+					    const guchar *image_data,
+					    gsize image_data_len)
+{
+    g_return_val_if_fail (track, FALSE);
+    g_return_val_if_fail (image_data, FALSE);
+
+    return gp_track_set_thumbnails_internal (track, NULL,
+					     image_data, image_data_len);
+}
+
+
 /* Set a thumbnail and store the filename in ExtraTrackData */
 gboolean gp_track_set_thumbnails (Track *track, const gchar *filename)
 {
-    gboolean result;
-    ExtraTrackData *etr;
-
     g_return_val_if_fail (track, FALSE);
     g_return_val_if_fail (filename, FALSE);
 
-    etr = track->userdata;
-    g_return_val_if_fail (etr, FALSE);
-
-    result = itdb_track_set_thumbnails (track, filename);
-    if (result == TRUE)
-    {
-	g_free (etr->thumb_path_locale);
-	g_free (etr->thumb_path_utf8);
-	etr->thumb_path_locale = g_strdup (filename);
-	etr->thumb_path_utf8 = charset_to_utf8 (filename);
-    }
-    else
-    {
-	g_free (etr->thumb_path_locale);
-	g_free (etr->thumb_path_utf8);
-	etr->thumb_path_locale = g_strdup ("");
-	etr->thumb_path_utf8 = g_strdup ("");
-	gtkpod_warning (_("Failed to set cover art: '%s'"), filename);
-    }
-    return result;
+    return gp_track_set_thumbnails_internal (track, filename,
+					     NULL, 0);
 }
 
+
 /* Remove a thumbnail and remove the filename in ExtraTrackData */
 /* Return value:
    FALSE: track did not have any thumbnails, so no change was done
@@ -346,8 +428,7 @@ gboolean gp_track_remove_thumbnails (Track *track)
     return changed;
 }
 
-
-/* add itdb to itdbs */
+/* add itdb to itdbs (and add to display) */
 void gp_itdb_add (iTunesDB *itdb, gint pos)
 {
     ExtraiTunesDBData *eitdb;
@@ -362,25 +443,17 @@ void gp_itdb_add (iTunesDB *itdb, gint pos)
     pm_add_itdb (itdb, pos);
 }
 
-/* return the podcast itdb */
-iTunesDB *gp_itdb_get_podcast ()
+/* Remove itdb to itdbs (and remove from display). Call
+ * itdb_free() to free the memory of the itdb. */
+void gp_itdb_remove (iTunesDB *itdb)
 {
-    GList *gl;
+    g_return_if_fail (itdbs_head);
+    g_return_if_fail (itdb);
 
-    g_return_val_if_fail (itdbs_head, NULL);
-
-    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
-    {
-	iTunesDB *itdb = gl->data;
-	g_return_val_if_fail (itdb, NULL);
-
-	if (itdb->usertype & GP_ITDB_TYPE_PODCASTS)
-	    return itdb;
-    }
-    return NULL;
+    pm_remove_playlist (itdb_playlist_mpl (itdb), FALSE);
+    itdbs_head->itdbs = g_list_remove (itdbs_head->itdbs, itdb);
 }
 
-
 /* Also replaces @old_itdb in the itdbs GList and take care that the
  * displayed itdb gets replaced as well */
 void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
@@ -389,7 +462,6 @@ void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
     Playlist *old_pl, *mpl;
     GList *old_link;
     gchar *old_pl_name = NULL;
-    GtkTreePath *path;
     gint pos = -1; /* default: add to the end */
 
     g_return_if_fail (old_itdb);
@@ -412,26 +484,17 @@ void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
     }
 
     /* get position of @old_itdb */
-    mpl = itdb_playlist_mpl (old_itdb);
-    g_return_if_fail (mpl);
-    path = pm_get_path (mpl);
-    if (path)
-    {
-	gint *indices = gtk_tree_path_get_indices (path);
-	if (indices)
-	    pos = indices[0];
-	gtk_tree_path_free (path);
-    }
+    pos = pm_get_position_for_itdb (old_itdb);
 
     /* remove @old_itdb (all playlists are removed if the MPL is
        removed and add @new_itdb at its place */
-    pm_remove_playlist (mpl, FALSE);
+    pm_remove_playlist (itdb_playlist_mpl (old_itdb), FALSE);
 
     /* replace old_itdb with new_itdb */
     new_eitdb->itdbs_head = itdbs_head;
     old_link->data = new_itdb;
     /* free old_itdb */
-    itdb_free (old_itdb);
+    gp_itdb_free (old_itdb);
 
     /* display replacement */
     pm_add_itdb (new_itdb, pos);
@@ -442,11 +505,13 @@ void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
 	Playlist *pl = itdb_playlist_by_name (new_itdb, old_pl_name);
 	if (pl) pm_select_playlist (pl);
     }
+
+    /* Set prefs system with name of MPL */
+    mpl = itdb_playlist_mpl (new_itdb);
+    set_itdb_prefs_string (new_itdb, "name", mpl->name);
 }    
 
 
-
-
 /* add playlist to itdb and to display */
 void gp_playlist_add (iTunesDB *itdb, Playlist *pl, gint32 pos)
 {
@@ -488,7 +553,7 @@ Playlist *gp_playlist_by_name_or_add (iTunesDB *itdb, gchar *pl_name,
     g_return_val_if_fail (pl_name, pl);
     pl = itdb_playlist_by_name (itdb, pl_name);
     if (pl)
-    {   /* check if the it's the same type (spl or normal) */
+    {   /* check if it's the same type (spl or normal) */
 	if (pl->is_spl == spl) return pl;
     }
     /* Create a new playlist */
@@ -543,7 +608,7 @@ guint gp_playlist_remove_by_name (iTunesDB *itdb, gchar *pl_name)
    No action is taken if "track" is not in the playlist.
    If "plitem" == NULL, remove from master playlist.
    If the track is removed from the MPL, it's also removed
-   from memory completely (i.e. from the tracklist and md5 hash).
+   from memory completely (i.e. from the tracklist and sha1 hash).
    Depending on @deleteaction, the track is either marked for deletion
    on the ipod/hard disk or just removed from the database
  */
@@ -563,10 +628,8 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
     case DELETE_ACTION_IPOD:
     case DELETE_ACTION_LOCAL:
     case DELETE_ACTION_DATABASE:
-	/* remove from MPL in these cases (unless we are removing
-	   podcasts) */
-	if (!(plitem && itdb_playlist_is_podcasts (plitem)))
-	    plitem = NULL;
+	/* remove from MPL in these cases */
+	plitem = NULL;
 	break;
     case DELETE_ACTION_PLAYLIST:
 	/* cannot remove from MPL */
@@ -587,6 +650,8 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
     /* remove track from playlist */
     itdb_playlist_remove_track (plitem, track);
 
+#if 0
+    /* podcasts are no longer treated differently from other playlists */
     /* if we removed a podcasts, remove it from memory as well, unless
        it's present in the MPL (this happens if this podcast was on
        the iPod as podcast as well as standard track) */
@@ -608,6 +673,7 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 	    gp_track_set_flags_default (track);
 	}
     }
+#endif
 
     if (itdb_playlist_is_mpl (plitem))
     { /* if it's the MPL, we remove the track permanently */
@@ -631,7 +697,6 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 
     if (remove_track)
     {
-	md5_track_remove (track);
 	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
 	{
 	    switch (deleteaction)
@@ -687,6 +752,8 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
    playlist @pl. It then lets the display model know.
    @display: if TRUE, track is added the display.  Otherwise it's only
    added to memory */
+/* All tracks added to the podcast playlist will get the mark_unplayed
+   flag get set */
 void gp_playlist_add_track (Playlist *pl, Track *track, gboolean display)
 {
     iTunesDB *itdb;
@@ -696,7 +763,12 @@ void gp_playlist_add_track (Playlist *pl, Track *track, gboolean display)
     itdb = pl->itdb;
     g_return_if_fail (itdb);
 
-    pl->members = g_list_append (pl->members, track);
+    itdb_playlist_add_track (pl, track, -1);
+    if (itdb_playlist_is_podcasts (pl))
+    {   /* have the iPod display a bullet in front of the track as it
+	   has been newly added */
+	track->mark_unplayed = 0x02;
+    }
     if (display)  pm_add_track (pl, track, TRUE);
 
     data_changed (itdb);
@@ -705,7 +777,7 @@ void gp_playlist_add_track (Playlist *pl, Track *track, gboolean display)
 
 /* Make sure all strings are initialised -- that way we don't
    have to worry about it when we are handling the strings.
-   exception: md5_hash, hostname and charset: these may be NULL. */
+   exception: sha1_hash, hostname and charset: these may be NULL. */
 void gp_track_validate_entries (Track *track)
 {
     ExtraTrackData *etr;
@@ -728,6 +800,16 @@ void gp_track_validate_entries (Track *track)
     if (!track->podcastrss)      track->podcastrss = g_strdup ("");
     if (!track->subtitle)        track->subtitle = g_strdup ("");
     if (!track->ipod_path)       track->ipod_path = g_strdup ("");
+    if (!track->tvshow)          track->tvshow = g_strdup("");
+    if (!track->tvepisode)       track->tvepisode = g_strdup("");
+    if (!track->tvnetwork)       track->tvnetwork = g_strdup("");
+    if (!track->albumartist)     track->albumartist = g_strdup ("");
+    if (!track->sort_artist)     track->sort_artist = g_strdup ("");
+    if (!track->sort_title)      track->sort_title = g_strdup ("");
+    if (!track->sort_album)      track->sort_album = g_strdup ("");
+    if (!track->sort_albumartist)track->sort_albumartist = g_strdup ("");
+    if (!track->sort_composer)   track->sort_composer = g_strdup ("");
+    if (!track->sort_tvshow)     track->sort_tvshow = g_strdup ("");
     if (!etr->pc_path_utf8)      etr->pc_path_utf8 = g_strdup ("");
     if (!etr->pc_path_locale)    etr->pc_path_locale = g_strdup ("");
     if (!etr->thumb_path_utf8)   etr->thumb_path_utf8 = g_strdup ("");
@@ -750,7 +832,7 @@ void gp_track_validate_entries (Track *track)
 void init_data (GtkWidget *window)
 {
     gchar *cfgdir;
-    gint type;
+    gint i;
 
     g_return_if_fail (window);
     g_return_if_fail (itdbs_head == NULL);
@@ -761,301 +843,214 @@ void init_data (GtkWidget *window)
 
     g_object_set_data (G_OBJECT (window), "itdbs_head", itdbs_head);
 
-    if (prefs_get_int_value ("itdb_0_type", &type))
-    {   /* databases have been set up previously -- use what is stored
-	 * in the prefs */
-	gint i;
-	for (i=0;;++i)
-	{
-	    gchar *property = g_strdup_printf ("itdb_%d_type", i);
-	    gboolean valid = prefs_get_int_value (property, &type);
-	    g_free (property);
-	    if (valid)
-	    {
-		iTunesDB *itdb = NULL;
-		Playlist *pl = NULL;
-		ExtraiTunesDBData *eitdb;
-		gchar *filename = NULL;
-		gchar *mountpoint = NULL;
-		gchar *offline_filename = NULL;
+    if (!prefs_get_int_value ("itdb_0_type", NULL))
+    {
+	/* databases have not been set up previously -- take care of
+	   this */
+#ifndef HAVE_GNOME_VFS
+	gchar *mountpoint;
+#endif
+	gchar *filename;
 
-		if (type & GP_ITDB_TYPE_LOCAL)
-		{
-		    gchar *fn = g_strdup_printf ("itdb_%d_filename", i);
+	/* Local database */
+	filename = g_build_filename (cfgdir, "local_0.itdb", NULL);
+	prefs_set_int ("itdb_0_type", GP_ITDB_TYPE_LOCAL);
+	prefs_set_string ("itdb_0_name", _("Local"));
+	prefs_set_string ("itdb_0_filename", filename);
+	g_free (filename);
 
-		    filename = prefs_get_string (fn);
+	/* Podcasts database */
+	filename = g_build_filename (cfgdir, "podcasts.itdb", NULL);
+	prefs_set_int ("itdb_1_type",
+		       GP_ITDB_TYPE_PODCASTS|GP_ITDB_TYPE_LOCAL);
+	prefs_set_string ("itdb_1_name", _("Podcasts"));
+	prefs_set_string ("itdb_1_filename", filename);
+	g_free (filename);
 
-		    if (!filename)
-		    {
-			gchar *local = g_strdup_printf ("local%d.itdb",i);
-			filename = g_build_filename (cfgdir, local, NULL);
-			g_free (local);
-		    }
-		    g_free (fn);
-		    if (g_file_test (filename, G_FILE_TEST_EXISTS))
-			itdb = gp_import_itdb (NULL, type,
-					       NULL, NULL, filename);
-		}
-		else if (type & GP_ITDB_TYPE_IPOD)
-		{
-		    gchar *mp = g_strdup_printf ("itdb_%d_mountpoint", i);
-		    gchar *fn = g_strdup_printf ("itdb_%d_filename", i);
-		    
-
-		    offline_filename = prefs_get_string (fn);
-		    mountpoint = prefs_get_string (mp);
-
-		    if (!offline_filename)
-		    {
-			gchar *local = g_strdup_printf ("gtkpod_%d.itdb",i);
-			offline_filename = g_build_filename (
-			    cfgdir, local, NULL);
-			g_free (local);
-		    }
-		    if (!mountpoint)
-			mountpoint = g_strdup (prefs_get_ipod_mount ());
-		    g_free (fn);
-		    g_free (mp);
-		}
-		else
-		{
-		    g_return_if_reached ();
-		}
-
-
-		if (!itdb)
-		{
-		    gchar *nm, *name;
-
-		    itdb = gp_itdb_new ();
-		    eitdb = itdb->userdata;
-		    g_return_if_fail (eitdb);
-		    itdb->usertype = type;
-		    itdb->filename = filename;
-		    itdb_set_mountpoint (itdb, mountpoint);
-		    g_free (mountpoint);
-		    eitdb->offline_filename = offline_filename;
-
-		    nm = g_strdup_printf ("itdb_%d_name", i);
-		    if (!prefs_get_string_value (nm, &name))
-		    {
-			if (type & GP_ITDB_TYPE_PODCASTS)
-			    name = g_strdup (_("Podcasts"));
-			else if (type & GP_ITDB_TYPE_LOCAL)
-			    name = g_strdup (_("Local"));
-			else
-			    name = g_strdup ("iPod");
-		    }
-		    pl = gp_playlist_new (name, FALSE);
-		    g_free (name);
-		    g_free (nm);
-		    itdb_playlist_set_mpl (pl);
-		    itdb_playlist_add (itdb, pl, -1);
-
-		    eitdb->data_changed = FALSE;
-		    eitdb->itdb_imported = FALSE;
-		}
-		else
-		{
-		    g_free (filename);
-		    g_free (mountpoint);
-		    g_free (offline_filename);
-		}
-
-		/* Check if Podcast Playlist is present on IPOD itdb
-		 * and add if not. If Podcast Playlist is present on
-		 * local itdb remove it. */
-		pl = itdb_playlist_podcasts (itdb);
-		if ((type & GP_ITDB_TYPE_IPOD) && !pl)
-		{   /* add podcast playlist */
-		    pl = gp_playlist_new (_("Podcasts"), FALSE);
-		    itdb_playlist_set_podcasts (pl);
-		    itdb_playlist_add (itdb, pl, -1);
-		    eitdb = itdb->userdata;
-		    g_return_if_fail (eitdb);
-		    eitdb->data_changed = FALSE;
-		}
-		if ((type & GP_ITDB_TYPE_LOCAL) && pl)
-		{   /* Remove podcast playlist. Normally no playlist
-		     * should be present, except for a few people who
-		     * used the CVS version between September and
-		     * October 2005. */
-		    if (itdb_playlist_tracks_number (pl) == 0)
-		    {
-			gp_playlist_remove (pl);
-		    }
-		    else
-		    {   /* OK, let's be nice and just drop the
-			   'podcast' flag instead of removing */
-			pl->podcastflag = 0;
-		    }
-		}
-		/* add to the display */
-		gp_itdb_add (itdb, -1);
-	    }
-	    else
-	    {
-		break; /* for (i=0;;++i) */
-	    }
-	}
+#ifndef HAVE_GNOME_VFS
+	/* iPod database -- only set up if autodetection is not active */
+	mountpoint = prefs_get_string ("initial_mountpoint");
+	filename = g_build_filename (cfgdir, "iTunesDB", NULL);
+	prefs_set_int ("itdb_2_type", GP_ITDB_TYPE_IPOD);
+	prefs_set_string ("itdb_2_name", _("iPod"));
+	prefs_set_string ("itdb_2_filename", filename);
+	prefs_set_string ("itdb_2_mountpoint", mountpoint);
+	g_free (mountpoint);
+	g_free (filename);
+#endif
     }
-    else
-    {   /* first run -- set up a local database and an iPod database */
-	iTunesDB *itdb;
-	ExtraiTunesDBData *eitdb;
-	gchar *fn;
-	Playlist *pl;
 
-	/* iPod database */
-	itdb = gp_itdb_new ();
+
+    for (i=0;;++i)
+    {
+	ExtraiTunesDBData *eitdb;
+	iTunesDB *itdb = setup_itdb_n (i);
+
+	if (itdb == NULL)
+	    break;
+
+	/* add to the display */
+	gp_itdb_add (itdb, -1);
+
+	/* update/sync playlists according to options set */
 	eitdb = itdb->userdata;
 	g_return_if_fail (eitdb);
-	itdb->usertype = GP_ITDB_TYPE_IPOD;
-	itdb_set_mountpoint (itdb, prefs_get_ipod_mount ());
-	eitdb->offline_filename = g_build_filename (
-	    cfgdir, "iTunesDB", NULL);
-	gp_itdb_add (itdb, -1);
-	pl = gp_playlist_new ("iPod", FALSE);
-	itdb_playlist_set_mpl (pl);   /* MPL! */
-	gp_playlist_add (itdb, pl, -1);
-	g_return_if_fail (itdb->userdata);
-	eitdb->data_changed = FALSE;
-
-	/* local database. First check if a database file already
-	   exists -- if yes load it */
-	itdb = NULL;
-	fn = g_build_filename (cfgdir, "local_0.itdb", NULL);
-	if (g_file_test (fn, G_FILE_TEST_EXISTS))
+	if (eitdb->itdb_imported)
 	{
-	    itdb = gp_import_itdb (NULL, GP_ITDB_TYPE_LOCAL,
-				   NULL, NULL, fn);
-	}
+	    /* take care of autosync... */
+	    sync_all_playlists (itdb);
 
-	if (!itdb)
-	{   /* local database does not exist or cannot be loaded */
-	    itdb = gp_itdb_new ();
-	    eitdb = itdb->userdata;
-	    g_return_if_fail (eitdb);
-	    itdb->usertype = GP_ITDB_TYPE_LOCAL;
-	    itdb->filename = g_strdup (fn);
-	    pl = gp_playlist_new (_("Local"), FALSE);
-	    itdb_playlist_set_mpl (pl);   /* MPL! */
-	    itdb_playlist_add (itdb, pl, -1);
+	    /* update all live SPLs */
+	    itdb_spl_update_live (itdb);
 	}
-
-	gp_itdb_add (itdb, -1);
-	g_free (fn);
     }
 
-    /* Add poscast itdb if not present */
-    if (!gp_itdb_get_podcast ())
-    {
-	iTunesDB *itdb;
-	ExtraiTunesDBData *eitdb;
-	gchar *fn;
-	Playlist *pl;
-
-	/*First check if a database file already exists -- if yes load
-	   it */
-	itdb = NULL;
-	fn = g_build_filename (cfgdir, "podcasts.itdb", NULL);
-	if (g_file_test (fn, G_FILE_TEST_EXISTS))
-	{
-	    itdb = gp_import_itdb (NULL, GP_ITDB_TYPE_PODCASTS|GP_ITDB_TYPE_LOCAL,
-				   NULL, NULL, fn);
-	}
-
-	if (!itdb)
-	{   /* local database does not exist or cannot be loaded */
-	    itdb = gp_itdb_new ();
-	    eitdb = itdb->userdata;
-	    g_return_if_fail (eitdb);
-	    itdb->usertype = GP_ITDB_TYPE_PODCASTS|GP_ITDB_TYPE_LOCAL;
-	    itdb->filename = g_strdup (fn);
-	    pl = gp_playlist_new (_("Podcasts"), FALSE);
-	    itdb_playlist_set_mpl (pl);   /* MPL! */
-	    itdb_playlist_add (itdb, pl, -1);
-	}
-
-	gp_itdb_add (itdb, -1);
-	g_free (fn);
-    }
-
-    /* set md5 */
-    prefs_set_md5tracks (prefs_get_md5tracks ());
-
-    /* update prefs with new information in case it's needed */
-    gp_update_itdb_prefs ();
-
     g_free (cfgdir);
 }
 
 
-/* Update the information about the itdbs in the prefs structure */
-void gp_update_itdb_prefs (void)
+
+/* Create an repository according to the settings in the preferences
+   system. */
+iTunesDB *setup_itdb_n (gint i)
 {
-    struct itdbs_head *itdbs_head;
-    GList *gl;
-    gint i=0;
-
-    g_return_if_fail (gtkpod_window);
-    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
-				    "itdbs_head");
-    g_return_if_fail (itdbs_head);
-    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
+    iTunesDB *itdb = NULL;
+    gchar *property = get_itdb_prefs_key (i, "type");
+    gint type;
+    gboolean valid = prefs_get_int_value (property, &type);
+    g_free (property);
+    if (valid)
     {
-	gchar *prop, *prop2;
-	Playlist *mpl;
+	gchar *cfgdir = prefs_get_cfgdir ();
+	Playlist *pl = NULL;
 	ExtraiTunesDBData *eitdb;
-	iTunesDB *itdb = gl->data;
+	gchar *filename = NULL;
+	gchar *mountpoint = NULL;
+	gchar *offline_filename = NULL;
 
-	g_return_if_fail (itdb);
-	eitdb = itdb->userdata;
-	g_return_if_fail (eitdb);
-
-	prop = g_strdup_printf ("itdb_%d_type", i);
-	prefs_set_int_value (prop, itdb->usertype);
-	g_free (prop);
-
-	mpl = itdb_playlist_mpl (itdb);
-	g_return_if_fail (mpl);
-	prop = g_strdup_printf ("itdb_%d_name", i);
-	prefs_set_string_value (prop, mpl->name);
-	g_free (prop);
-
-	prop = g_strdup_printf ("itdb_%d_filename", i);
-	prop2 = g_strdup_printf ("itdb_%d_mountpoint", i);
-	if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	if (type & GP_ITDB_TYPE_LOCAL)
 	{
-	    prefs_set_string_value (prop, itdb->filename);
-	    prefs_set_string_value (prop2, NULL);
+	    gchar *fn = get_itdb_prefs_key (i, "filename");
+
+	    filename = prefs_get_string (fn);
+
+	    if (!filename)
+	    {
+		gchar *local = g_strdup_printf ("local%d.itdb",i);
+		filename = g_build_filename (cfgdir, local, NULL);
+		g_free (local);
+	    }
+	    g_free (fn);
+	    if (g_file_test (filename, G_FILE_TEST_EXISTS))
+		itdb = gp_import_itdb (NULL, type,
+				       NULL, NULL, filename);
 	}
-	else if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	else if (type & GP_ITDB_TYPE_IPOD)
 	{
-	    prefs_set_string_value (prop, eitdb->offline_filename);
-	    prefs_set_string_value (prop2, itdb->mountpoint);
+	    gchar *key;
+
+	    key = get_itdb_prefs_key (i, KEY_MOUNTPOINT);
+	    mountpoint = prefs_get_string (key);
+	    g_free (key);
+
+	    key = get_itdb_prefs_key (i, "filename");
+	    offline_filename = prefs_get_string (key);
+	    g_free (key);
+
+	    if (!offline_filename)
+	    {
+		gchar *local = g_strdup_printf ("gtkpod_%d.itdb",i);
+		offline_filename = g_build_filename (
+		    cfgdir, local, NULL);
+		g_free (local);
+	    }
 	}
 	else
 	{
-	    g_return_if_reached ();
+	    g_return_val_if_reached (NULL);
 	}
-	g_free (prop);
-	g_free (prop2);
 
-	++i;
+	if (!itdb)
+	{
+	    gchar *nm, *name;
+
+	    itdb = gp_itdb_new ();
+	    eitdb = itdb->userdata;
+	    g_return_val_if_fail (eitdb, NULL);
+	    itdb->usertype = type;
+	    itdb->filename = filename;
+	    itdb_set_mountpoint (itdb, mountpoint);
+	    eitdb->offline_filename = offline_filename;
+
+	    nm = g_strdup_printf ("itdb_%d_name", i);
+	    if (!prefs_get_string_value (nm, &name))
+	    {
+		if (type & GP_ITDB_TYPE_PODCASTS)
+		    name = g_strdup (_("Podcasts"));
+		else if (type & GP_ITDB_TYPE_LOCAL)
+		    name = g_strdup (_("Local"));
+		else
+		    name = g_strdup (_("iPod"));
+	    }
+	    pl = gp_playlist_new (name, FALSE);
+	    g_free (name);
+	    g_free (nm);
+	    itdb_playlist_set_mpl (pl);
+	    itdb_playlist_add (itdb, pl, -1);
+
+	    eitdb->data_changed = FALSE;
+	    eitdb->itdb_imported = FALSE;
+	}
+	else
+	{
+	    g_free (filename);
+	    g_free (offline_filename);
+	}
+	g_free (mountpoint);
+
+	/* Check if Podcast Playlist is present on IPOD itdb
+	 * and add if not. If Podcast Playlist is present on
+	 * local itdb remove it. */
+	pl = itdb_playlist_podcasts (itdb);
+	if ((type & GP_ITDB_TYPE_IPOD) && !pl)
+	{   /* add podcast playlist */
+	    pl = gp_playlist_new (_("Podcasts"), FALSE);
+	    itdb_playlist_set_podcasts (pl);
+	    itdb_playlist_add (itdb, pl, -1);
+	    eitdb = itdb->userdata;
+	    g_return_val_if_fail (eitdb, NULL);
+	    eitdb->data_changed = FALSE;
+	}
+	if ((type & GP_ITDB_TYPE_LOCAL) && pl)
+	{   /* Remove podcast playlist. Normally no playlist
+	     * should be present, except for a few people who
+	     * used the CVS version between September and
+	     * October 2005. */
+	    if (itdb_playlist_tracks_number (pl) == 0)
+	    {
+		gp_playlist_remove (pl);
+	    }
+	    else
+	    {   /* OK, let's be nice and just drop the
+		   'podcast' flag instead of removing */
+		pl->podcastflag = 0;
+	    }
+	}
+	g_free (cfgdir);
     }
+    return itdb;
 }
 
 
-/* Increase playcount of filename <file> by <num>. If md5 is activated,
-   use md5 to find the track. Otherwise use the filename. If @md5 is
+/* Increase playcount of filename <file> by <num>. If sha1 is activated,
+   use sha1 to find the track. Otherwise use the filename. If @sha1 is
    set, this value is used directly to look up the track in the
    database (instead of calculating it from the file).
 
    Return value:
    TRUE: OK (playcount increased in GP_ITDB_TYPE_IPOD)
    FALSE: file could not be found in the GP_ITDB_TYPE_IPOD */
-gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num)
+gboolean gp_increase_playcount (gchar *sha1, gchar *file, gint num)
 {
     gboolean result = FALSE;
     Track *track = NULL;
@@ -1068,19 +1063,18 @@ gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num)
 	iTunesDB *itdb = gl->data;
 	g_return_val_if_fail (itdb, FALSE);
 
-	if (md5) track = md5_md5_exists (itdb, md5);
-	else     track = md5_file_exists (itdb, file, TRUE);
+	if (sha1) track = sha1_sha1_exists (itdb, sha1);
+	else     track = sha1_file_exists (itdb, file, TRUE);
 	if (!track)	  track = gp_track_by_filename (itdb, file);
 	if (track)
 	{
-	    gchar *buf1, *buf;
+	    gchar *buf1;
 	    track->playcount += num;
 	    data_changed (itdb);
 	    pm_track_changed (track);
 	    buf1 = get_track_info (track, TRUE);
-	    buf = g_strdup_printf (_("Increased playcount for '%s'"), buf1);
-	    gtkpod_statusbar_message (buf);
-	    g_free (buf);
+	    gtkpod_statusbar_message (_("Increased playcount for '%s'"),
+				      buf1);
 	    g_free (buf1);
 	    if (itdb->usertype & GP_ITDB_TYPE_IPOD)    result = TRUE;
 	}
@@ -1089,12 +1083,11 @@ gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num)
 }
 
 
-/* determine "active" itdb -- it's either the itdb of the playlist
- * currently selected, or the first itdb if none is selected */
-iTunesDB *gp_get_active_itdb (void)
+/* get the currently selected itdb. NULL is
+ * returned if no itdb is active. */
+iTunesDB *gp_get_selected_itdb (void)
 {
     Playlist *pl = pm_get_selected_playlist ();
-    struct itdbs_head *itdbs_head;
 
     /* If playlist is selected, use the itdb of the playlist as the
        active itdb */
@@ -1104,34 +1097,69 @@ iTunesDB *gp_get_active_itdb (void)
 	return pl->itdb;
     }
 
-    /* Otherwise choose the first itdb */
-    g_return_val_if_fail (gtkpod_window, NULL);
-    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
-				    "itdbs_head");
-    g_return_val_if_fail (itdbs_head, NULL);
-    g_return_val_if_fail (itdbs_head->itdbs, NULL);
-    g_return_val_if_fail (itdbs_head->itdbs->data, NULL);
-    return itdbs_head->itdbs->data;
+    return NULL;
 }
 
 
-/* get the "ipod" itdb, that's the first itdb with
-   type==GP_ITDB_TYPE_IPOD. Returns NULL and prints error when no
-   matching itdb can be found */
+/* Get the "ipod" itdb. If only one iPod itdb exists, this itdb is
+ * returned. If more than one iPod itdb exists, the currently selected
+ * itdb is returned if it's an iPod itdb, otherwise NULL is returned.
+ */
 iTunesDB *gp_get_ipod_itdb (void)
 {
     struct itdbs_head *itdbs_head;
+    iTunesDB *itdb;
     GList *gl;
+    gint i;
+
+    /* if an iPod itdb is selected, return this */
+    itdb = gp_get_selected_itdb ();
+    if (itdb && (itdb->usertype & GP_ITDB_TYPE_IPOD))
+	return itdb;
+
+    itdb = NULL;
 
     g_return_val_if_fail (gtkpod_window, NULL);
     itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
 				    "itdbs_head");
+
+    if (itdbs_head == NULL) return NULL;
+
+    i=0;
+    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
+    {
+	iTunesDB *itdbgl = gl->data;
+	g_return_val_if_fail (itdbgl, NULL);
+	if (itdbgl->usertype & GP_ITDB_TYPE_IPOD)
+	{
+	    itdb = itdbgl;
+	    ++i;
+	}
+    }
+    /* return iPod itdb if only one was found */
+    if (i == 1)
+	return itdb;
+
+    return NULL;
+}
+
+
+/* return the podcast itdb */
+iTunesDB *gp_get_podcast_itdb ()
+{
+    GList *gl;
+
     g_return_val_if_fail (itdbs_head, NULL);
+
     for (gl=itdbs_head->itdbs; gl; gl=gl->next)
     {
 	iTunesDB *itdb = gl->data;
 	g_return_val_if_fail (itdb, NULL);
-	if (itdb->usertype & GP_ITDB_TYPE_IPOD)  return itdb;
+
+	if (itdb->usertype & GP_ITDB_TYPE_PODCASTS)
+	    return itdb;
     }
-    g_return_val_if_reached (NULL);
+    return NULL;
 }
+
+
