@@ -1,4 +1,4 @@
-/* Time-stamp: <2006-03-04 11:00:52 jcs>
+/* Time-stamp: <2007-02-24 13:22:58 jcs>
 |
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
@@ -25,25 +25,26 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file_export.c,v 1.58 2006/03/04 02:15:46 jcsjcs Exp $
+|  $Id: file_export.c 997 2007-02-24 05:48:33Z jcsjcs $
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include "misc.h"
-#include "prefs.h"
-#include "file.h"
 #include "charset.h"
+#include "file.h"
 #include "info.h"
-#include "md5.h"
-#include <limits.h>
-#include <stdio.h>
+#include "sha1.h"
+#include "misc.h"
+#include "misc_track.h"
+#include "prefs.h"
 #include <errno.h>
+#include <limits.h>
+#include <math.h>
+#include <stdio.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <string.h>
-#include <math.h>
 #include <unistd.h>
 
 /* Structure to keep all necessary information */
@@ -285,11 +286,13 @@ static gboolean write_track (struct fcd *fcd)
 	gchar *from_file = NULL;
 	if (fcd->track->itdb->usertype & GP_ITDB_TYPE_IPOD)
 	{
-	    from_file = get_file_name_on_ipod (fcd->track);
+	    from_file = get_file_name_from_source (fcd->track,
+						   SOURCE_IPOD);
 	}
 	else if (fcd->track->itdb->usertype & GP_ITDB_TYPE_LOCAL)
 	{
-	    from_file = get_file_name_on_harddisk (fcd->track);
+	    from_file = get_file_name_from_source (fcd->track,
+						   SOURCE_LOCAL);
 	}
 	else
 	{
@@ -320,8 +323,10 @@ static gboolean write_track (struct fcd *fcd)
 	}
 	else
 	{
+	    gchar *buf = get_track_info (fcd->track, FALSE);
 	    gtkpod_warning (_("Could find file for '%s' on the iPod\n"),
-			    get_track_info (fcd->track, FALSE));
+			    buf);
+	    g_free (buf);
 	}
 	g_free(dest_file);
     }
@@ -431,8 +436,8 @@ static void export_files_write (struct fcd *fcd)
 	start = time(NULL);
 	for(l = fcd->tracks; l && !abort; l = l->next)
 	{
+	    gboolean resultWrite = TRUE;
 	    Track *tr = (Track*)l->data;
-	    gchar *buf;
 
 	    fcd->track = tr;
 	    copied += tr->size;
@@ -452,32 +457,38 @@ static void export_files_write (struct fcd *fcd)
 		    g_cond_timed_wait (cond, mutex, &gtime);
 		} while(!mutex_data);
 		g_mutex_unlock (mutex);
-		result &= (gboolean)GPOINTER_TO_INT(g_thread_join (thread));
+		resultWrite = (gboolean)GPOINTER_TO_INT(g_thread_join (thread));
+		result &= resultWrite;
 	    }
 	    else {
 		g_warning ("Thread creation failed, falling back to default.\n");
-		result &= write_track (fcd);
+		resultWrite = write_track (fcd);
+		result &= resultWrite;
 	    }
 #else
-	    result &= write_track (fcd);
+	    resultWrite = write_track (fcd);
+	    result &= resultWrite;
 	    while (widgets_blocked && gtk_events_pending ())
 		gtk_main_iteration ();
 #endif
-	    if (!result)
+	    if (!resultWrite)
 	    {
 		gtkpod_warning (_("Failed to write '%s-%s'\n"), tr->artist, tr->title);	
 	    }
 
 	    ++count;
-	    if (count == 1)  /* we need longer timeout */
-		prefs_set_statusbar_timeout (3*STATUSBAR_TIMEOUT);
+	    if (count == 1) /* we need longer timeout */
+	    {
+		gtkpod_statusbar_timeout (3*STATUSBAR_TIMEOUT);
+	    }
+	        
 	    if (count == n)  /* we need to reset timeout */
-		prefs_set_statusbar_timeout (0);
-	    buf = g_strdup_printf (ngettext ("Copied %d of %d track.",
-					     "Copied %d of %d tracks.", n),
-				   count, n);
-	    gtkpod_statusbar_message(buf);
-	    g_free (buf);
+	    {
+		gtkpod_statusbar_timeout (0);
+	    }
+	    gtkpod_statusbar_message (ngettext ("Copied %d of %d track.",
+						"Copied %d of %d tracks.", n),
+				      count, n);
 
 	    fraction = copied/total;
 
@@ -543,14 +554,22 @@ void export_files_init (GList *tracks, GList **filenames,
     struct fcd *fcd;
     GtkWidget *fc;
     GladeXML *export_files_xml;
+    iTunesDB *itdb = NULL;
 
-    /* no export possible if in offline mode */
-    if (tracks && prefs_get_offline ())
+    if (tracks)
     {
 	Track *tr = tracks->data;
 	g_return_if_fail (tr);
-	g_return_if_fail (tr->itdb);
-	if (tr->itdb->usertype & GP_ITDB_TYPE_IPOD)
+	itdb = tr->itdb;
+	g_return_if_fail (itdb);
+    }
+
+    /* no export possible if in offline mode */
+    if (tracks && get_offline (itdb))
+    {
+	Track *tr = tracks->data;
+	g_return_if_fail (tr);
+	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
 	{
 	    GtkWidget *dialog = gtk_message_dialog_new (
 		GTK_WINDOW (gtkpod_window),
@@ -697,8 +716,8 @@ GList *export_trackglist_when_necessary (iTunesDB *itdb_s,
     g_return_val_if_fail (itdb_d, NULL);
     g_return_val_if_fail (gtkpod_window, NULL);
 
-    if (!(itdb_s->usertype & GP_ITDB_TYPE_IPOD) ||
-	!(itdb_d->usertype & GP_ITDB_TYPE_LOCAL))
+    if (!((itdb_s->usertype & GP_ITDB_TYPE_IPOD) &&
+	  (itdb_d->usertype & GP_ITDB_TYPE_LOCAL)))
     {   /* drag is not from iPod to local database -> return copy of
 	 * @tracks */
 	return g_list_copy (tracks);
@@ -707,7 +726,7 @@ GList *export_trackglist_when_necessary (iTunesDB *itdb_s,
     /* drag is from iPod to local database */
 
     /* no drag possible if in offline mode */
-    if (prefs_get_offline ())
+    if (get_offline (itdb_s))
     {
 	GtkWidget *dialog = gtk_message_dialog_new (
 	    GTK_WINDOW (gtkpod_window),
@@ -728,7 +747,7 @@ GList *export_trackglist_when_necessary (iTunesDB *itdb_s,
 	Track *tr = gl->data;
 	g_return_val_if_fail (tr, NULL);
 
-	otr = md5_track_exists (itdb_d, tr);
+	otr = sha1_track_exists (itdb_d, tr);
 
 	if (otr)
 	{
@@ -924,9 +943,9 @@ static void export_playlist_file_write (struct fcd *fcd)
 
     num = g_list_length (fcd->tracks);
 
-    prefs_get_int_value (EXPORT_PLAYLIST_FILE_TYPE, &type);
-    prefs_get_int_value (EXPORT_PLAYLIST_FILE_SOURCE, &source);
-    prefs_get_string_value (EXPORT_PLAYLIST_FILE_TPL, &template);
+    type = prefs_get_int (EXPORT_PLAYLIST_FILE_TYPE);
+    source = prefs_get_int (EXPORT_PLAYLIST_FILE_SOURCE);
+    template = prefs_get_string (EXPORT_PLAYLIST_FILE_TPL);
     if (!template)
 	template = g_strdup (EXPORT_PLAYLIST_FILE_TPL_DFLT);
 
@@ -937,7 +956,6 @@ static void export_playlist_file_write (struct fcd *fcd)
     if (file)
     {
 	guint i,n;
-	gchar *buf;
 
 	switch (type)
 	{
@@ -1004,11 +1022,9 @@ static void export_playlist_file_write (struct fcd *fcd)
 	    break;
 	}
 	fclose (file);
-	buf = g_strdup_printf (
+	gtkpod_statusbar_message  (
 	    ngettext ("Created playlist with one track.",
 		      "Created playlist with %d tracks.", n), n);
-	gtkpod_statusbar_message (buf);
-	g_free (buf);
     }
     else
     {
